@@ -1,4 +1,6 @@
-fRegress <- function(yfdPar, xfdlist, betalist, wt=rep(1,N)) {
+fRegress <- function(y, xfdlist, betalist, wt=NULL, 
+                     y2cMap=NULL, SigmaE=NULL) 
+{
 
 #  FREGRESS  Fits a functional linear model using multiple
 #  functional independent variables with the dependency being
@@ -8,7 +10,7 @@ fRegress <- function(yfdPar, xfdlist, betalist, wt=rep(1,N)) {
 #  and a unit coefficient.
 #
 #  Arguments:
-#  YFDPAR   ... an object for the dependent variable,
+#  Y        ... an object for the dependent variable,
 #               which may be:
 #                   a functional data object,
 #                   a functional parameter (fdPar) object, or
@@ -28,6 +30,13 @@ fRegress <- function(yfdPar, xfdlist, betalist, wt=rep(1,N)) {
 #               if BETALIST is a functional parameter object
 #               it is converted to a list of length 1.
 #  WT       ... a vector of nonnegative weights for observations
+#  Y2CMAP   ... the matrix mapping from the vector of observed values
+#               to the coefficients for the dependent variable.
+#               This is output by function SMOOTH_BASIS.  If this is
+#               supplied, confidence limits are computed, otherwise not.
+#  SIGMAE   ... Estimate of the covariances among the residuals.  This
+#               can only be estimated after a preliminary analysis
+#               with FREGRESS.
 #
 #  Returns FREGRESSLIST  ... A list containing seven members with names:
 #    yfdPar      ... first  argument of FREGRESS
@@ -35,127 +44,28 @@ fRegress <- function(yfdPar, xfdlist, betalist, wt=rep(1,N)) {
 #    betalist    ... third  argument of FREGRESS
 #    betaestlist ... estimated regression functions
 #    yhatfdobj   ... functional data object containing fitted functions
+#    Cmat        ... coefficient matrix for the linear system defining
+#                    the regression coefficient basis vector
+#    Dmat        ... right side vector for the linear system defining
+#                    the regression coefficient basis vector
 #    Cmatinv     ... inverse of the coefficient matrix, needed for
 #                    function FREGRESS.STDERR that computes standard errors
 #    wt          ... weights for observations
 #    df          ... degrees of freedom for fit
 
-#  Last modified 21 October 2008 by Jim
+#  Last modified 29 October 2009 by Jim Ramsay
 
-#  check YFDPAR and compute sample size N
+arglist <- fRegressArgCheck(y, xfdlist, betalist, wt)
 
-if (inherits(yfdPar, "fd")) yfdPar <- fdPar(yfdPar)
-
-if (!(inherits(yfdPar, "fdPar") || inherits(yfdPar, "numeric"))) stop(
-  "First argument is not of class 'fd', 'fdPar' or 'numeric'.")
-
-if (inherits(yfdPar, "fdPar")) {
-    yfd   <- yfdPar$fd
-    ycoef <- yfd$coefs
-    N     <- dim(ycoef)[2]
-}
-if (inherits(yfdPar, "numeric")) {
-    N <- length(yfdPar)
-}
-
-#  get number of independent variables p
+yfdPar   <- arglist$yfdPar
+xfdlist  <- arglist$xfdlist
+betalist <- arglist$betalist
+wt       <- arglist$wt 
 
 p <- length(xfdlist)
+N <- dim(xfdlist[[1]]$coef)[1]
 
-#  check BETALIST
-
-if (inherits(betalist, "fd")) betalist <- list(betalist)
-
-if (!inherits(betalist, "list")) stop(
-	"Argument BETALIST is not a list object.")
-
-if (length(betalist) != p)  {
-	cat(paste("\nNumber of regression coefficients does not match\n",
-		       "number of independent variables."))
-	stop("")
-}
-
-berror <- FALSE
-for (j in 1:p) {
-	betafdParj <- betalist[[j]]
-	if (inherits(betafdParj, "fd")) {
-		betafdParj    <- fdPar(betafdParj)
-		betalist[[j]] <- betafdParj
-	}
-	if (!inherits(betafdParj, "fdPar")) {
-		print(paste("BETALIST[[",j,"]] is not a FDPAR object."))
-		berror <- TRUE
-	}
-}
-
-if (berror) stop("")
-
-betafd1    <- betalist[[1]]$fd
-betabasis1 <- betafd1$basis
-rangeval   <- betabasis1$rangeval
-
-#  check XFDLIST
-
-if (inherits(xfdlist, "fd") || inherits(xfdlist, "numeric")) 
-    xfdlist <- list(xfdlist)
-
-if (!inherits(xfdlist, "list")) stop(
-	"Argument XFDLIST is not a list object.")
-
-#  check each xfdlist member.  If the object is a vector of length N,
-#  it is converted to a functional data object with a
-#  constant basis
-
-onebasis <- create.constant.basis(rangeval)
-onesfd   <- fd(1,onebasis)
-
-xerror <- FALSE
-for (j in 1:p) {
-    xfdj <- xfdlist[[j]]
-    if (inherits(xfdj, "fd")) {
-        xcoef <- xfdj$coefs
-        if (length(dim(xcoef)) > 2) stop(
-            paste("Covariate",j,"is not univariate."))
-        #  check size of coefficient array
-        Nj    <- dim(xcoef)[2]
-        if (Nj != N) {
-            print(
-               paste("Incorrect number of replications in XFDLIST",
-                     "for covariate",j))
-            xerror = TRUE
-        }
-    } 
-    if (inherits(xfdj, "numeric")) {
-        if (!is.matrix(xfdj)) xfdj = as.matrix(xfdj)
-	  Zdimj <- dim(xfdj)
-        if (Zdimj[1] != N) {
-            print(paste("Vector in XFDLIST[[",j,"]] has wrong length."))
-				    xerror = TRUE 
-		    } 
-        if (Zdimj[2] != 1) {
-            print(paste("Matrix in XFDLIST[[",j,"]] has more than one column."))
-				    xerror = TRUE 
-		    } 
-        xfdlist[[j]] <- fd(matrix(xfdj,1,N), onebasis)
-    } 
-    if (!(inherits(xfdlist[[j]], "fd") || 
-          inherits(xfdlist[[j]], "numeric"))) {
-      print(paste("XFDLIST[[",j,"]] is neither an FD object nor numeric."))
-      xerror = TRUE
-    }
-}
-    
-if (xerror) stop("")        
-
-#  check weights
-
-if (length(wt) != N) stop("Number of weights not equal to N.")
-if (any(wt < 0))     stop("Negative weights found.")
-if (length(wt) == 1 || var(wt) > 0) {
-    wtconstant <- F 
-} else {
-    wtconstant <- T
-}
+wtconst <- var(wt) == 0
 
 #  --------------------------------------------------------------------------
 #  branch depending on whether the dependent variable is functional or scalar 
@@ -181,9 +91,7 @@ if (inherits(yfdPar, "fdPar")) {
 
     if (length(ycoefdim) > 2) stop("YFDOBJ from YFDPAR is not univariate.")
 
-    #  -----------------------------------------------------------
-    #          set up the linear equations for the solution
-    #  -----------------------------------------------------------
+    #  --------  set up the linear equations for the solution  -----------
 
     #  compute the total number of coefficients to be estimated
 
@@ -242,7 +150,8 @@ if (inherits(yfdPar, "fdPar")) {
                         xxfdjk <- (xfdj*wt)*xfdk
                     }
                     wtfdjk <- sum(xxfdjk)
-                    Cmatjk <- inprod(betabasisj, betabasisk, 0, 0, rangeval, wtfdjk)
+                    Cmatjk <- inprod(betabasisj, betabasisk, 0, 0, rangeval, 
+                                     wtfdjk)
                     Cmat[indexj,indexk] <- Cmatjk
                     Cmat[indexk,indexj] <- t(Cmatjk)
                 }
@@ -278,12 +187,12 @@ if (inherits(yfdPar, "fdPar")) {
     for (j in 1:p) {
         betafdParj <- betalist[[j]]
         if (betafdParj$estimate) {
-            betafdj    <- betafdParj$fd
-            ncoefj     <- betafdj$basis$nbasis
-            mj1    <- mj2 + 1
-            mj2    <- mj2 + ncoefj
-            indexj <- mj1:mj2
-            coefj  <- betacoef[indexj]
+            betafdj <- betafdParj$fd
+            ncoefj  <- betafdj$basis$nbasis
+            mj1     <- mj2 + 1
+            mj2     <- mj2 + ncoefj
+            indexj  <- mj1:mj2
+            coefj   <- betacoef[indexj]
             betafdj$coefs <- as.matrix(coefj)
             betafdParj$fd <- betafdj
         }
@@ -305,16 +214,35 @@ if (inherits(yfdPar, "fdPar")) {
     }
     yhatfdobj <- smooth.basis(tfine, yhatmat, ybasisobj)$fd
 
-    #  set up output list object
+    df <- NA
+    
+    #  -----------------------------------------------------------------------
+    #        Compute pointwise standard errors of regression coefficients
+    #               if both y2cMap and SigmaE are supplied.
+    #  -----------------------------------------------------------------------
+
+ 
+
+    #  -------------------------------------------------------------------
+    #                       Set up output list object
+    #  -------------------------------------------------------------------
 
     fRegressList <-
-       	list(yfdPar      = yfdPar,
-                 xfdlist     = xfdlist,
-                 betalist    = betalist,
-                 betaestlist = betaestlist,
-                 yhatfdobj   = yhatfdobj,
-                 Cmatinv     = Cmatinv,
-                 wt          = wt)
+       	list(yfdPar         = yfdPar,
+                 xfdlist        = xfdlist,
+                 betalist       = betalist,
+                 betaestlist    = betaestlist,
+                 yhatfdobj      = yhatfdobj,
+                 Cmat           = Cmat,
+                 Dmat           = Dmat,
+                 Cmatinv        = Cmatinv,
+                 wt             = wt,
+                 df             = df,
+                 y2cMap         = y2cMap,
+                 SigmaE         = SigmaE,
+                 betastderrlist = betastderrlist,
+                 bvar           = bvar,
+                 c2bMap         = c2bMap)
 
  }
 
@@ -388,35 +316,25 @@ if (inherits(yfdPar, "fdPar")) {
 
     #  compute and print degrees of freedom measure
 
-    hatvals = diag(Zmat %*% Cmatinv %*% t(Zmat))	
-    df <- sum(hatvals)
+    df <- sum(diag(Zmat %*% Cmatinv %*% t(Zmat)))
 
     #  set up fdPar object for BETAESTFDPAR
 
     betaestlist <- betalist
-    onebasis    <- create.constant.basis(c(0,1))
     mj2 <- 0
     for (j in 1:p) {
         betafdParj <- betalist[[j]]
         betafdj    <- betafdParj$fd
         ncoefj     <- betafdj$basis$nbasis
-        mj1    <- mj2 + 1
-        mj2    <- mj2 + ncoefj
-        indexj <- mj1:mj2
-        betacoefj <- betacoef[indexj]
-        if (inherits(xfdj, "fd")) {
-            betaestfdj       <- betafdj
-            betaestfdj$coefs <- as.matrix(betacoefj)
-            betaestfdParj    <- betafdParj
-            betaestfdParj$fd <- betaestfdj
-            betaestlist[[j]] <- betaestfdParj
-        }
-        else{
-            betaestfdj       <- fd(as.matrix(t(betacoefj)),onebasis)
-            betaestfdParj    <- betafdParj
-            betaestfdParj$fd <- betaestfdj
-            betaestlist[[j]] <- betaestfdParj
-        }
+        mj1        <- mj2 + 1
+        mj2        <- mj2 + ncoefj
+        indexj     <- mj1:mj2
+        betacoefj        <- betacoef[indexj]
+        betaestfdj       <- betafdj
+        betaestfdj$coefs <- as.matrix(betacoefj)
+        betaestfdParj    <- betafdParj
+        betaestfdParj$fd <- betaestfdj
+        betaestlist[[j]] <- betaestfdParj
     }
 
     #  set up fd object for predicted values
@@ -435,37 +353,82 @@ if (inherits(yfdPar, "fdPar")) {
             betafdParj <- betaestlist[[j]]
             betafdj    <- betafdParj$fd
             betamat    <- eval.fd(tfine, betafdj)
-            fitj       <- deltat*
-				(crossprod(xmat,betamat) -
-						0.5*(outer(xmat[1,    ],betamat[1,    ]) +
-				            outer(xmat[nfine,],betamat[nfine,])))
+            fitj       <- deltat*(crossprod(xmat,betamat) -
+						                 0.5*(outer(xmat[1,    ],betamat[1,    ]) +
+				                          outer(xmat[nfine,],betamat[nfine,])))
             yhatmat    <- yhatmat + fitj
-        }
-        else{
-	       betaestfdParj <- betaestlist[[j]]
+        } else{
+	          betaestfdParj <- betaestlist[[j]]
             betavecj      <- betaestfdParj$fd$coefs
             yhatmat       <- yhatmat + xfdj %*% t(betavecj)
         }
     }
     yhatfdobj <- yhatmat
 
-    OCV = sum( (ymat-yhatmat)^2/(1-hatvals)^2 )
+    #  -----------------------------------------------------------------------
+    #        Compute pointwise standard errors of regression coefficients
+    #               if both y2cMap and SigmaE are supplied.
+    #  -----------------------------------------------------------------------
 
+    if (!(is.null(y2cMap) || is.null(SigmaE))) {
     
+        #  check dimensions of y2cMap and SigmaE
+
+        y2cdim <- dim(y2cMap)
+        if (y2cdim[2] != dim(SigmaE)[1])  stop(
+                         "Dimensions of Y2CMAP not correct.")
 
 
-    #  set up output list object
+        #  compute linear mapping c2bMap takinging coefficients for
+        #  response into coefficients for regression functions
+
+        c2bMap <- Cmatinv %*% t(Zmat)
+        y2bmap <- c2bMap
+        bvar   <- y2bmap %*% as.matrix(SigmaE) %*% t(y2bmap)
+        betastderrlist <- vector("list",p)
+        mj2 <- 0
+        for (j in 1:p) {
+	      betafdParj <- betalist[[j]]
+            betabasisj <- betafdParj$fd$basis
+	      ncoefj     <- betabasisj$nbasis
+            mj1        <- mj2 + 1
+            mj2        <- mj2 + ncoefj
+            indexj     <- mj1:mj2
+            bvarj      <- bvar[indexj,indexj]
+            betarng    <- betabasisj$rangeval
+            nfine      <- max(c(501,10*ncoefj+1))
+            tfine      <- seq(betarng[1], betarng[2], len=nfine)
+            bbasismat  <- eval.basis(tfine, betabasisj)
+            bstderrj   <- sqrt(diag(bbasismat %*% bvarj %*% t(bbasismat)))
+            bstderrfdj <- smooth.basis(tfine, bstderrj, betabasisj)$fd
+            betastderrlist[[j]] <- bstderrfdj
+        }
+    } else {
+        betastderrlist = NULL
+        bvar           = NULL
+        c2bMap         = NULL
+    }
+                         
+    #  -----------------------------------------------------------------------
+    #                  Set up output list object
+    #  -----------------------------------------------------------------------
 
     fRegressList <-
-       	list(yfdPar      = yfdPar,
-                 xfdlist     = xfdlist,
-                 betalist    = betalist,
-                 betaestlist = betaestlist,
-                 yhatfdobj   = yhatfdobj,
-                 Cmatinv     = Cmatinv,
-                 wt          = wt,
-                 df          = df,
-		     OCV         = OCV)
+       	list(yfdPar         = yfdPar,
+                 xfdlist        = xfdlist,
+                 betalist       = betalist,
+                 betaestlist    = betaestlist,
+                 yhatfdobj      = yhatfdobj,
+                 Cmat           = Cmat,
+                 Dmat           = Dmat,
+                 Cmatinv        = Cmatinv,
+                 wt             = wt,
+                 df             = df,
+                 y2cMap         = y2cMap,
+                 SigmaE         = SigmaE,
+                 betastderrlist = betastderrlist,
+                 bvar           = bvar,
+                 c2bMap         = c2bMap)
  }
 
  return(fRegressList)
@@ -492,7 +455,8 @@ eigchk <- function(Cmat) {
     logcondition <- log10(eigval[1]) - log10(eigval[ncoef])
     if (logcondition > 12) {
         warning("Near singularity in coefficient matrix.")
-        cat(paste("\nEigenvalues range from",eigval[ncoef]," to ",eigval[1]))
+        cat(paste("\nLog10 Eigenvalues range from\n",
+                  log10(eigval[ncoef])," to ",log10(eigval[1]),"\n"))
     }
 }
 
